#include <sys/stat.h>
#include <libgen.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include "parse_mode.h"

static char v_flag;
static char p_flag;
static mode_t mode = 0777;

static int do_mkdir(const char *path) {
	if (mkdir(path, mode)) {
		if (p_flag)
			return 0;

		fprintf(stderr, "mkdir: %s: %s\n", path, strerror(errno));
		return 1;
	}

	else if (v_flag)
		printf("mkdir: %s created with mode %u\n", path, mode);

	return 0;
}

static int do_parents(const char *path) {
	if (path[0] == '.' || path[0] == '/')
		return 0;

	char *path2 = strdup(path);
	if (!path2) {
		fprintf(stderr, "mkdir: %s: %s\n", path, strerror(errno));
		return 1;
	}

	const char *par = dirname(path2);
	do_parents(par);
	int ret = do_mkdir(path2);

	free(path2);
	return ret;
}

int main(int argc, char **argv) {
	int opt;
	while ((opt = getopt(argc, argv, "pm:v")) != -1) {
		switch (opt) {
			case 'p':
				p_flag = 1;
				break;

			case 'm':
				mode = mu_parse_mode(optarg, 0);
				break;

			case 'v':
				v_flag = 1;
				break;

			default:
				puts("mkdir [pmv] [dst1 dst2...]\n\t-p Make parent dir\n\t-m MODE Mode\n\t-v Verbose");
				return 0;
		}
	}

	argv += optind;
	argc -= optind;

	for (int i = 0; i < argc; i++) {
		if (p_flag) {
			if (do_parents(argv[i]))
				return 1;
		}

		else {
			if (do_mkdir(argv[i]))
				return 1;
		}
	}

	return 0;
}
