#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <pwd.h>
#include <time.h>
#include "proc_parser.h"
#include "human.h"

static int pscan(const pid_t pid) {
	struct mu_proc proc;
	if (mu_proc_parser("ps", pid, &proc))
		return 1;

	/* Uid */
	char *name = "unknow";
	struct passwd *pw = getpwuid(proc.uid);
	if (pw != NULL)
		name = pw->pw_name;

	/* Time */
	unsigned int rtime = (proc.utime + proc.stime) / sysconf(_SC_CLK_TCK);

	/* Print */
	char virt[MU_HUMAN_BUF_SIZE + 1];
	strcpy(virt, mu_humansize((off_t)proc.vsize, 1024));

	char rss[MU_HUMAN_BUF_SIZE + 1];
	strcpy(rss, mu_humansize((off_t)proc.vmrss * 1024, 1024));

	printf("%6d %8s %4ld %4ld %8s %8s %2c %02um:%02us %2s\n", proc.pid, name, proc.priority, proc.nice, virt, rss, proc.state, rtime / 60, rtime % 60, proc.prog);
	return 0;
}

int main(int argc, char **argv) {
	while (getopt(argc, argv, "") != -1) {
		puts("ps [a] [PID]\n\t-a Print all processes\n");
		return 0;
	}

	argv += optind;
	argc -= optind;

	puts("   PID     USER  PRI NICE     VIRT      RSS  S RTIME   CMD");

	int ret = 0;
	if (argc == 0) {
		DIR *dp = opendir("/proc");
		if (dp == NULL) {
			fprintf(stderr, "ps: /proc: %s\n", strerror(errno));
			return 1;
		}

		struct dirent *ep;
		while ((ep = readdir(dp)) != NULL) {
			pid_t pid = strtoul(ep->d_name, 0L, 10);
			if (pid)
				if (pscan(pid))
					ret = 1;
		}

		closedir(dp);
	}

	else {
		for (int i = 0; i < argc; i++) {
			pid_t pid = strtoul(argv[i], 0L, 10);
			if (pid)
				if (pscan(pid))
					ret = 1;
		}
	}

	return ret;
}
